@@ -1,40 +1,58 @@
-from typing import Any
+from typing import Any, Union
 
 from hatchling.builders.hooks.plugin.interface import BuildHookInterface
 from typing import BinaryIO
-from tempfile import NamedTemporaryFile
+from tempfile import mkstemp
 from pathlib import Path
+import os
+from subprocess import run, DEVNULL
+
+class UnopenedTemporaryFile:
+    '''Like NamedTemporaryFile, but not open by default and only with functionality that we need.
+    '''
+    def __init__(self, *args, **kwargs):
+        handle, pathname = mkstemp(*args, **kwargs)
+        os.close(handle)
+        self.name: Union[str, bytes] = pathname
+
+    def __str__(self) -> str:
+        if isinstance(self.name, str):
+            return self.name
+        else:
+            return os.fsdecode(self.name)
+
+    def __bytes__(self) -> bytes:
+        if isinstance(self.name, bytes):
+            return self.name
+        else:
+            return os.fsencode(self.name)
+
+    def __del__(self):
+        os.remove(self.name)
+
+
+class MsgfmtBuildHook(BuildHookInterface):
+    PLUGIN_NAME = 'msgfmt'
 
-class BuildHookBase(BuildHookInterface):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.__files = []
 
-    def convert(self, input: Path) -> bytes:
-        raise NotImplementedError()
-
     def initialize(self, version: str, build_data: dict[str, Any]) -> None:
         if self.target_name != 'wheel':
             return
 
-        for file in self.build_config.builder.recurse_included_files():
-            path = Path(file.path)
-            relative_path = Path(file.relative_path)
-            distribution_path = Path(file.distribution_path)
+        for included in self.build_config.builder.recurse_included_files():
+            path = Path(included.path)
+            distribution_path = Path(included.distribution_path)
             if distribution_path.suffix == '.po':
-                file = NamedTemporaryFile(mode='wb', suffix='.mo')
+                output = UnopenedTemporaryFile(suffix='.mo')
 
-                file.write(self.convert(path))
-                file.flush()
+                run(['msgfmt', '-o', str(output.name), included.path], check=True, stdin=DEVNULL)
 
-                self.__files.append(file)
-                build_data['force_include'][file.name] = str(distribution_path.with_suffix('.mo'))
-                print(f'doing {path}')
-
-        if 'exclude' not in build_data:
-            build_data['exclude'] = []
-
-        build_data['exclude'].append('*.po')
+                self.__files.append(output)
+                build_data['force_include'][output.name] = str(distribution_path.with_suffix('.mo'))
+                path.unlink()
 
         print(f'{[(file.path, file.relative_path, file.distribution_path) for file in  self.build_config.builder.recurse_included_files()]=}')
 
